<?php

namespace FiloBlu\ActiveCampaign\Model\Connector;

use FiloBlu\ActiveCampaign\Helper\Data;
use GuzzleHttp\Client;
use GuzzleHttp\ClientFactory;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\ResponseFactory;
use Magento\Framework\DataObjectFactory;
use Psr\Log\LoggerInterface;
use Magento\Framework\Webapi\Rest\Request;

class Rest extends AbstractConnector {
    /**
     * @var ClientFactory
     */
    private $clientFactory;
    /**
     * @var ResponseFactory
     */
    private $responseFactory;
    /**
     * @var
     */
    private $connection;

    /**
     * Rest constructor.
     * @param ClientFactory $clientFactory
     * @param ResponseFactory $responseFactory
     * @param Data $helperData
     */
    public function __construct(
        ClientFactory $clientFactory,
        ResponseFactory $responseFactory,
        Data $helperData,
        DataObjectFactory $dataObjectFactory,
        LoggerInterface $logger
    ) {
        $this->clientFactory = $clientFactory;
        $this->responseFactory = $responseFactory;
        parent::__construct($helperData,$dataObjectFactory,$logger);
    }

    /**
     * @return ConnectorInterface
     */
    public function connect(): ConnectorInterface
    {
        if (!$this->connection) {
            $this->connection = $this->clientFactory->create(['config' => [
                'base_uri' => $this->helperData->getRestBaseUrl(),
                'headers' => [
                    'Api-Token' => $this->helperData->getRestToken()
                ]
            ]]);
        }

        return $this;
    }

    /**
     * @return ConnectionInterface
     */
    public function disconnect(): ConnectionInterface
    {
        return $this;
    }

    /**
     * @param string $uriEndpoint
     * @param array $params
     * @param string $requestMethod
     * @return Response
     */
    public function doRequest(
        string $uriEndpoint,
        array $params = [],
        string $requestMethod = Request::HTTP_METHOD_GET
    )
    {
        try {
            $response = $this->connection->request(
                $requestMethod,
                $uriEndpoint,
                $params
            );
        } catch (GuzzleException $exception) {
            /** @var Response $response */
            $response = $this->responseFactory->create([
                'status' => $exception->getCode(),
                'reason' => $exception->getMessage()
            ]);
        }

        return $response;
    }
}